/* Board */
#include "board.h"

/* App */
#include "main.h"

#define APP_BPC_TIM_CHANNEL TIM_CHANNEL_1

int app_bpc_init(void) {
    /* Enable BPC module */
    HAL_GPIO_WritePin(BOARD_BPC_PON_PORT, BOARD_BPC_PON_PIN, 0U);

    htim14.Instance               = TIM14;
    htim14.Init.Prescaler         = (HAL_RCC_GetPCLK1Freq() / 1000) - 1;
    htim14.Init.Period            = 2500 - 1;
    htim14.Init.RepetitionCounter = 0;
    htim14.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_ENABLE;
    htim14.Init.ClockDivision     = TIM_CLOCKDIVISION_DIV1;
    htim14.Init.CounterMode       = TIM_COUNTERMODE_UP;

    if (HAL_TIM_Base_Init(&htim14) != HAL_OK) {
        return -1;
    }

    TIM_IC_InitTypeDef ic_init = {
        .ICSelection = TIM_ICSELECTION_DIRECTTI,
        .ICPolarity  = TIM_ICPOLARITY_BOTHEDGE,
        .ICPrescaler = TIM_ICPSC_DIV1,
        .ICFilter    = 0,
    };

    if (HAL_TIM_IC_ConfigChannel(&htim14, &ic_init, APP_BPC_TIM_CHANNEL) != HAL_OK) {
        return -2;
    }

    /* Enable reload interrupt, for timeout detection */
    __HAL_TIM_ENABLE_IT(&htim14, TIM_IT_UPDATE);

    if (HAL_TIM_IC_Start_IT(&htim14, APP_BPC_TIM_CHANNEL) != HAL_OK) {
        return -3;
    }

    return 0;
}

void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim) {
    if (htim->Instance == TIM14) {
        HAL_GPIO_TogglePin(BOARD_LED_2_PORT, BOARD_LED_2_PIN);
    }
}

void HAL_TIM_IC_CaptureCallback(TIM_HandleTypeDef *htim) {
    if (htim->Instance == TIM14) {
        __HAL_TIM_SET_COUNTER(htim, 0U);

        HAL_GPIO_TogglePin(BOARD_LED_1_PORT, BOARD_LED_1_PIN);
    }
}